<?php

namespace Drupal\ro_dashboard\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\link\LinkItemInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the Multi Site Status Dashboard Site.
 *
 * @ingroup ro_dashboard
 *
 * @ContentEntityType(
 *   id = "ro_dashboard_site",
 *   label = @Translation("Monitored website"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\ro_dashboard\RoDashboardSiteListBuilder",
 *     "access" = "Drupal\ro_dashboard\RoDashboardSiteAccessControlHandler",
 *     "permission_provider" = "Drupal\Core\Entity\EntityPermissionProvider",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "default" = "Drupal\ro_dashboard\Form\RoDashboardSiteForm",
 *       "add" = "Drupal\ro_dashboard\Form\RoDashboardSiteForm",
 *       "edit" = "Drupal\ro_dashboard\Form\RoDashboardSiteForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "revision-delete" = \Drupal\Core\Entity\Form\RevisionDeleteForm::class,
 *       "revision-revert" = \Drupal\Core\Entity\Form\RevisionRevertForm::class
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *       "revision" = \Drupal\Core\Entity\Routing\RevisionHtmlRouteProvider::class,
 *     },
 *   },
 *   base_table = "ro_dashboard_site",
 *   revision_table = "ro_dashboard_site_revision",
 *   admin_permission = "administer ro_dashboard sites",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision_id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *     "published" = "status",
 *   },
 *   links = {
 *     "canonical" = "/ro-dashboard/site/{ro_dashboard_site}",
 *     "add-form" = "/ro-dashboard/sites/add",
 *     "edit-form" = "/ro-dashboard/sites/{ro_dashboard_site}/edit",
 *     "delete-form" = "/ro-dashboard/sites/{ro_dashboard_site}/delete",
 *     "revision" = "/ro-dashboard/sites/{ro_dashboard_site}/revision/{ro_dashboard_site_revision}/view",
 *     "revision-delete-form" = "/ro-dashboard/sites/{ro_dashboard_site}/revision/{ro_dashboard_site_revision}/delete",
 *     "revision-revert-form" = "/ro-dashboard/sites/{ro_dashboard_site}/revision/{ro_dashboard_site_revision}/revert",
 *     "version-history" = "/ro-dashboard/sites/{ro_dashboard_site}/revisions",
 *   },
 * )
 */
class RoDashboardSite extends ContentEntityBase implements RoDashboardSiteInterface {

  use EntityChangedTrait;
  use EntityOwnerTrait;
  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    $global_status = $this->get('global_status');
    $status = [];
    foreach ($global_status as $value) {
      $status[] = $value->value;
    }
    if (in_array('STATUS_ERROR', $status) ||
    in_array('SECURITY', $status) ||
    in_array('CONNECTION_ERROR', $status)) {
      $this->set('status_class', 'error');
    }
    elseif (in_array('UPDATES', $status) ||
    in_array('STATUS_WARNING', $status)) {
      $this->set('status_class', 'warning');
    }
    else {
      $this->set('status_class', 'ok');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime(): ?int {
    return $this->get('created')->value ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp): RoDashboardSite {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getChangedTime(): ?int {
    return $this->get('changed')->value ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setReport(array $report): RoDashboardSite {
    $this->set('collected_reports', serialize($report));
    $this->setNewRevision();
    $this->save();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getReport(): array {
    if ($this->get('collected_reports')->isEmpty()) {
      return [];
    }
    $reports = unserialize($this->get('collected_reports')->value, ['allowed_classes' => FALSE]);
    return !(empty($reports)) ? $reports : [];
  }

  /**
   * {@inheritdoc}
   */
  public function getSiteUrl(): string {
    $link = $this->get('site_url')->first();
    /** @var \Drupal\link\LinkItemInterface $link */
    return $link->getUrl()->toString() ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getSgStatusReportIgnore(): array {
    return $this->mergeIgnoreValues('sg_status_report_ignore', 'sg_status_report_ignore');
  }

  /**
   * {@inheritdoc}
   */
  public function getSgModuleUpdateReportIgnore(): array {
    return $this->mergeIgnoreValues('sg_module_update_report_ignore', 'sg_module_update_report_ignore');
  }

  /**
   * Merge the global and local ignore values.
   *
   * Both values are strings with newlines as separators. Explode these strings
   * and merge the arrays.
   *
   * @param string $global_key
   *   The key of global ignore values (config key).
   * @param string $local_key
   *   The key of local ignore values (entity field).
   *
   * @return array
   *   The array with merged ignore values.
   */
  private function mergeIgnoreValues(string $global_key, string $local_key): array {
    $config = \Drupal::config('ro_dashboard.settings');
    $global_ignore = $config->get($global_key) ?? '';
    $local_ignore = $this->get($local_key)->value ?? '';
    $global_values_to_ignore = preg_split("(\r\n?|\n)", $global_ignore);
    $local_values_to_ignore = preg_split("(\r\n?|\n)", $local_ignore);
    return array_merge($global_values_to_ignore, $local_values_to_ignore);
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['label'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Site name'))
      ->setDescription(t('The name of the site.'))
      ->setRequired(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 0,
      ]);

    $fields['site_url'] = BaseFieldDefinition::create('link')
      ->setLabel(t('Site URL'))
      ->setDescription(t('The URL of the site.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'link',
        'weight' => 0,
      ])
      ->setSettings([
        'title' => 0,
        'link_type' => LinkItemInterface::LINK_EXTERNAL,
      ]);
    ;

    $fields['drupal_version'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Drupal version'))
      ->setDescription(t('The installed Drupal version.'))
      ->setRequired(FALSE)
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'region' => 'hidden',
      ]);

    $fields['php_version'] = BaseFieldDefinition::create('string')
      ->setLabel(t('PHP version'))
      ->setDescription(t('The PHP version of the server.'))
      ->setRequired(FALSE)
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'region' => 'hidden',
      ]);

    $fields['global_status'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Global status'))
      ->setDescription(t('The site status after processing the reports.'))
      ->setRequired(FALSE)
      ->setRevisionable(TRUE)
      ->setSettings([
        'allowed_values' => [
          'OK' => 'All OK',
          'UPDATES' => 'Updates available',
          'SECURITY' => 'Security updates available',
          'STATUS_WARNING' => 'Warnings on status report',
          'STATUS_ERROR' => 'Errors on status report',
          'CONNECTION_ERROR' => 'Connection error',
        ],
      ])
      ->setCardinality(-1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
      ])
      ->setDisplayOptions('form', [
        'region' => 'hidden',
      ]);

    $fields['status_class'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Status class'))
      ->setDescription(t('Row class to reflect the site status.'));

    $fields['last_check'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Last checked'))
      ->setDescription(t('The date the site is last checked.'))
      ->setSettings([
        'datetime_type' => 'datetime',
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'datetime_default',
      ])
      ->setDisplayOptions('form', [
        'region' => 'hidden',
      ]);

    $fields['collected_reports'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Collected report'))
      ->setDescription(t('The data retrieved from sites.'))
      ->setSetting('case_sensitive', TRUE)
      ->setRevisionable(TRUE)
      ->setDefaultValue('')->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'ro_dashboard_data',
        'weight' => 10,
      ]);

    $fields['sg_token'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Site Guardian-token'))
      ->setDescription(t('The token that can be used to retrieve Site-Guardian report.'))
      ->setRequired(FALSE)
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'region' => 'hidden',
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 0,
      ]);

    $fields['sg_status_report'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Site Guardian status report'))
      ->setDescription(t('Collect and process the Site Guardian status report.'))
      ->setTranslatable(FALSE)
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
      ]);

    $fields['sg_status_report_ignore'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Ignored items'))
      ->setDescription(t('These values will not affect the site status. These
      values will be added to the global ignored values.'))
      ->setSetting('case_sensitive', TRUE)
      ->setRevisionable(TRUE)
      ->setDefaultValue('')->setDisplayOptions('view', [
        'region' => 'hidden',
      ])
      ->setDisplayOptions('form', [
        'type' => 'basic_string',
      ]);

    $fields['sg_module_update_report'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Site Guardian update report'))
      ->setDescription(t('Collect and process the Site Guardian module updates report.'))
      ->setTranslatable(FALSE)
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
      ]);

    $fields['sg_module_update_report_ignore'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Ignored items'))
      ->setDescription(t('These updates will not affect the site status. These
      values will be added to the global ignored values.'))
      ->setSetting('case_sensitive', TRUE)
      ->setRevisionable(TRUE)
      ->setDefaultValue('')->setDisplayOptions('view', [
        'region' => 'hidden',
      ])
      ->setDisplayOptions('form', [
        'type' => 'basic_string',
      ]);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Published'))
      ->setTranslatable(FALSE)
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 100,
      ]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time when the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time when the entity was last edited.'));

    return $fields;
  }

}
