<?php

namespace Drupal\ro_dashboard\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\ro_dashboard\Service\SiteGuardianService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin to correctly format the generated site-report.
 *
 * @FieldFormatter(
 *   id = "ro_dashboard_data",
 *   label = @Translation("Format collected site data"),
 *   field_types = {
 *     "string_long"
 *   }
 * )
 */
class SiteReportFormatter extends FormatterBase {

  /**
   * The service to format the SiteGuardian data.
   *
   * @var \Drupal\ro_dashboard\Service\SiteGuardianService
   */
  protected $siteGuardianService;

  /**
   * Constructs a SiteReportFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\ro_dashboard\Service\SiteGuardianService $site_guardian_service
   *   The service to parse the SiteGuardian data.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    SiteGuardianService $site_guardian_service,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->siteGuardianService = $site_guardian_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('ro_dashboard.site_guardian')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Correctly formats the serialized data for a site.');
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {
      $site = $item->getEntity();
      $data = $site->getReport();
      $parsed_data = [];

      $sg_status_report = $site->get('sg_status_report') ?? FALSE;
      if ($sg_status_report && !empty($data['sg_status'])) {
        $sg_status_report_ignore = $site->getSgStatusReportIgnore() ?? [];
        $parsed_data[] = $this->siteGuardianService->parseStatusReportForDisplay($data['sg_status'], $sg_status_report_ignore);
      }
      $sg_module_update_report = $site->get('sg_module_update_report') ?? FALSE;
      if ($sg_module_update_report && !empty($data['sg_modules'])) {
        $sg_module_update_report_ignore = $site->getSgModuleUpdateReportIgnore() ?? [];
        $parsed_data[] = $this->siteGuardianService->parseModuleUpdateReportForDisplay($data['sg_modules'], $sg_module_update_report_ignore);
      }

      $element[$delta] = [
        '#theme' => 'ro_dashboard_report',
        '#report_data' => $parsed_data,
      ];
      $element[$delta]['#attached']['library'][] = 'ro_dashboard/ro-dashboard-sites-table';
    }

    return $element;
  }

}
