<?php

namespace Drupal\ro_dashboard\Service;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\ClientInterface;

/**
 * Service to collect SiteGuardian data from a site.
 */
class SiteGuardianService {

  use StringTranslationTrait;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Drupal\Core\Render\RendererInterface definition.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannel
   */
  protected $loggerChannel;

  // Constants for the status report severity levels.
  const REQUIREMENT_INFO = -1;
  const REQUIREMENT_OK = 0;
  const REQUIREMENT_WARNING = 1;
  const REQUIREMENT_ERROR = 2;
  // Constants for the module status.
  const NOT_SECURE = 1;
  const REVOKED = 2;
  const NOT_SUPPORTED = 3;
  const NOT_CURRENT = 4;
  const CURRENT = 5;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ClientInterface $http_client,
    RendererInterface $renderer,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->httpClient = $http_client;
    $this->renderer = $renderer;
    $this->loggerChannel = $logger_factory->get('ro_dashboard');
  }

  /**
   * Lookup Site Guardian Status report.
   *
   * @param string $report_to_get
   *   The report to get: modules or status.
   * @param string $site_url
   *   The site URL.
   * @param string $site_token
   *   The site token.
   * @param array $ignore
   *   The ignore array.
   *
   * @return array
   *   Array with 'data' (array with the raw report) and 'info' (array with
   *   'warning', 'error', 'php_version' and 'drupal_version').
   */
  public function getReport(string $report_to_get, string $site_url, string $site_token, array $ignore) {
    $results = [
      'data' => [],
      'info' => [
        'connection' => 'OK',
      ],
    ];
    if (empty($site_url) || empty($site_token)) {
      return $results;
    }
    if ($report_to_get === 'modules') {
      $url_template = '%s/site_guardian/enabled_modules_and_updates?site_guardian_key=%s';
    }
    else {
      $url_template = '%s/site_guardian/status_report?site_guardian_key=%s';
    }
    $url = sprintf($url_template, $site_url, $site_token);

    try {
      $response = $this->httpClient->request(
        'GET', $url, [
          'headers' => [
            'Accept' => 'application/json',
          ],
        ]);
    }
    catch (\Exception $x) {
      $results['info']['connection'] = 'CONNECTION_ERROR';
      $this->loggerChannel
        ->error('Error connecting to the Site Guardian endpoint. Error: @error', [
          '@error' => $x->getMessage(),
        ]);
      return $results;
    }

    if ($response->getStatusCode() !== 200) {
      $results['info']['connection'] = 'CONNECTION_ERROR';
      $this->loggerChannel
        ->error('Error connecting to the Site Guardian endpoint. Status code: @status_code', [
          '@status_code' => $response->getStatusCode(),
        ]);
      return $results;
    }

    $data = json_decode($response->getBody()->getContents(), TRUE);
    $results['data'] = $data;
    if ($report_to_get === 'modules') {
      $results['info'] = $this->analyzeModuleUpdates($results['data'], $ignore);
    }
    else {
      $results['info'] = $this->analyzeStatus($results['data'], $ignore);
    }
    $results['info']['connection'] = 'OK';

    return $results;
  }

  /**
   * Analyze SiteGuardian status report data.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   * @param array $ignore
   *   Array with the ignore list.
   *
   * @return array
   *   Return number of errors, warnings, the Drupal- and PHP version numbers.
   */
  public function analyzeStatus($data, $ignore) {
    $results = [
      'warning' => 0,
      'error' => 0,
      'drupal_version' => '',
      'php_version' => '',
    ];
    foreach ($data as $key => $value) {
      $severity = $value['severity'] ?? self::REQUIREMENT_OK;
      if ($key === 'drupal') {
        $results['drupal_version'] = $value['value'] ?? '';
      }
      if ($key === 'php') {
        $php_info = $value['value'] ?? '';
        $php_array = explode(' ', $php_info);
        $php_version = $php_array[0] ?? '';
        $results['php_version'] = $php_version;
      }
      if (!in_array($key, $ignore)) {
        if ($severity === self::REQUIREMENT_WARNING) {
          $results['warning']++;
        }
        if ($severity === self::REQUIREMENT_ERROR) {
          $results['error']++;
        }
      }
    }
    return $results;
  }

  /**
   * Analyze SiteGuardian module updates report.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   * @param array $ignore
   *   Array with the ignore list.
   *
   * @return array
   *   Return number of errors, warnings.
   */
  public function analyzeModuleUpdates($data, $ignore) {
    $results = [
      'warning' => 0,
      'error' => 0,
    ];
    foreach ($data as $key => $value) {
      $status = $value['status'] ?? self::CURRENT;
      // Check if we should ignore this item.
      if (!in_array($key, $ignore)) {
        if ($status === self::NOT_CURRENT || $status === self::NOT_SUPPORTED) {
          $results['warning']++;
        }
        if ($status === self::REVOKED || $status === self::NOT_SECURE) {
          $results['error']++;
        }
      }
    }
    return $results;
  }

  /**
   * Parse the decoded JSON data and return a working render array.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   * @param array $ignore
   *   Array with the ignore list.
   *
   * @return array
   *   The render array.
   */
  public function parseStatusReportForDisplay($data, $ignore) {
    $severities = $this->getSeverities();
    $results = [];
    $ignored_title = $this->t('Ignored');
    // Parse the data.
    $header = [
      $this->t('Item'),
      $this->t('Details'),
      $this->t('Severity'),
    ];
    $rows = [
      'general' => [
        'others' => [],
        'warning' => [],
        'error' => [],
        'title' => $this->t('General'),
      ],
      'php_status' => [
        'others' => [],
        'warning' => [],
        'error' => [],
        'title' => $this->t('PHP status'),
      ],
      'server_benchmarks' => [
        'others' => [],
        'warning' => [],
        'error' => [],
        'title' => $this->t('Server benchmarks'),
      ],
      'user_status' => [
        'others' => [],
        'warning' => [],
        'error' => [],
        'title' => $this->t('Users'),
      ],
    ];
    $sub_report_php_modules = [];
    $sub_report_benchmarks = [];
    $sub_report_user_roles = [];
    $sub_report_watchdog = [];
    // Loop through the entire report.
    foreach ($data as $key => $value) {
      $ignore_item = in_array($key, $ignore);
      $title = $value['title'] ?? '';
      $value_safe = $value['value'] ?? '';
      $severity = $value['severity'] ?? self::REQUIREMENT_OK;
      $severity_label_suffix = $ignore_item ? ' (' . $ignored_title . ')' : '';
      $severity_label = ($severities[$severity]['title'] ?? '') . $severity_label_suffix;
      $severity_class = $ignore_item ? 'ignored' : ($severities[$severity]['status'] ?? '');
      $description = '';
      if (!empty($value['description'])) {
        if (is_array($value['description'])) {
          $description .= $this->renderer->render($value['description']);
        }
        else {
          $description .= '<br><em>' . $value['description'] . '</em>';
        }
        $description = '<span class="description">' . $description . '</span>';
      }
      $value_array = [
        '#markup' => $title . ': ' . $value_safe . $description,
      ];
      $report_type = 'general';
      $destination = 'others';

      if (strpos($key, 'sgd_php_extensions') === 0) {
        $sub_report_php_modules = $this->parsePhpModulesSubReportForDisplay($value);
        continue;
      }
      if (strpos($key, 'sgd_server_benchmarks') === 0) {
        $sub_report_benchmarks = $this->parseServerBenchmarksSubReportForDisplay($value);
        continue;
      }
      if (strpos($key, 'sgd_user_roles') === 0) {
        $sub_report_user_roles = $this->parseUserRolesSubReportForDisplay($value);
        continue;
      }
      if (strpos($key, 'sgd_watchdog_summary') === 0) {
        $sub_report_watchdog = $this->parseWatchdogSubReportForDisplay($value);
        continue;
      }
      if (strpos($key, 'php') === 0) {
        $report_type = 'php_status';
      }
      if (strpos($key, 'sgd_benchmarks') === 0) {
        $report_type = 'server_benchmarks';
      }
      if (strpos($key, 'sgd_user') === 0) {
        $report_type = 'user_status';
      }
      if ($severity === self::REQUIREMENT_WARNING) {
        $destination = 'warning';
      }
      if ($severity === self::REQUIREMENT_ERROR) {
        $destination = 'error';
      }
      $rows[$report_type][$destination][] = [
        'data' => [
          $key,
          $this->renderer->render($value_array),
          $severity_label,
        ],
        'class' => [
          'status-' . $severity_class,
        ],
      ];

    }
    foreach ($rows as $key => $value) {
      if (!empty($value['error']) || !empty($value['warning']) || !empty($value['others'])) {
        $extra = [];
        if ($key === 'php_status' && !empty($sub_report_php_modules)) {
          $extra = $sub_report_php_modules;
        }
        if ($key === 'server_benchmarks' && !empty($sub_report_benchmarks)) {
          $extra = $sub_report_benchmarks;
        }
        if ($key === 'user_status' && !empty($sub_report_user_roles)) {
          $extra = $sub_report_user_roles;
        }
        $results['status'][] = $this->generateResult($header, $value, [
          'report_title' => $value['title'],
          'error_title' => $this->t('Errors found'),
          'warning_title' => $this->t('Warnings found'),
          'others_title' => $this->t('Other status information'),
        ],
        $extra);
      }
    }
    if (!empty($sub_report_watchdog)) {
      $results['status'][] = $sub_report_watchdog;
    }

    return $results;
  }

  /**
   * Parse a sub-array of the php modules.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   *
   * @return array
   *   The render array.
   */
  public function parsePhpModulesSubReportForDisplay($data) {
    $results = [];
    // Parse the data.
    $header = [
      $this->t('Item'),
      $this->t('Version'),
    ];
    $rows = [
      'others' => [],
      'title' => $this->t('Enabled PHP extensions'),
    ];
    foreach ($data as $key => $value) {
      $rows['others'][] = [
        'data' => [
          $key,
          $value['version'],
        ],
        'class' => [
          'status-ok',
        ],
      ];

    }
    if (!empty($rows['others'])) {
      $results = $this->generateResult($header, $rows, [
        'report_title' => $rows['title'],
        'others_title' => $this->t('Enabled PHP extensions'),
      ]);
    }

    return $results;
  }

  /**
   * Parse a sub-array of the server benchmarks.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   *
   * @return array
   *   The render array.
   */
  public function parseServerBenchmarksSubReportForDisplay($data) {
    $results = [];
    // Parse the data.
    $header = [
      $this->t('Item'),
      $this->t('Result'),
      $this->t('Description'),
    ];
    $rows = [
      'others' => [],
      'title' => $this->t('Server benchmarks - details'),
    ];
    foreach ($data as $key => $value) {
      if ($key === 'settings') {
        continue;
      }
      $value_array = [
        '#markup' => '<strong>' . $key . ' - tests</strong>',
      ];
      $rows['others'][] = [
        'data' => [
          $this->renderer->render($value_array),
          '',
          '',
        ],
        'class' => [
          'status-ok',
        ],
      ];
      foreach ($value as $item => $value2) {
        if ($item === 'total') {
          continue;
        }
        $rows['others'][] = [
          'data' => [
            $item . ': ' . $value2['title'],
            $value2['result'],
            htmlspecialchars_decode($value2['description']),
          ],
          'class' => [
            'status-ok',
          ],
        ];
      }
    }
    if (!empty($rows['error']) || !empty($rows['warning']) || !empty($rows['others'])) {
      $results = $this->generateResult($header, $rows, [
        'report_title' => $rows['title'],
        'others_title' => $this->t('Server benchmarks - details'),
      ]);
    }

    return $results;
  }

  /**
   * Parse a sub-array of the role-statistics.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   *
   * @return array
   *   The render array.
   */
  public function parseUserRolesSubReportForDisplay($data) {
    $results = [];
    // Parse the data.
    $header = [
      $this->t('Role'),
      $this->t('Number of users'),
    ];
    $rows = [
      'others' => [],
      'title' => $this->t('Roles and number of users'),
    ];
    foreach ($data as $value) {
      $rows['others'][] = [
        'data' => [
          $value['label'] . ' (' . $value['roles_target_id'] . ')',
          $value['count'],
        ],
        'class' => [
          'status-ok',
        ],
      ];
    }
    if (!empty($rows['error']) || !empty($rows['warning']) || !empty($rows['others'])) {
      $results = $this->generateResult($header, $rows, [
        'report_title' => $rows['title'],
        'others_title' => $this->t('Roles and number of users'),
      ]);
    }

    return $results;
  }

  /**
   * Parse a sub-array of the watchdog report.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   *
   * @return array
   *   The render array.
   */
  public function parseWatchdogSubReportForDisplay($data) {
    $results = [];
    // Parse the data.
    $header = [
      $this->t('Type'),
      $this->t('Emergency'),
      $this->t('Alert'),
      $this->t('Critical'),
      $this->t('Error'),
      $this->t('Warning'),
      $this->t('Notice'),
      $this->t('Info'),
      $this->t('Debug'),
      $this->t('Total'),
    ];
    $rows = [
      'others' => [],
      'warning' => [],
      'error' => [],
      'title' => $this->t('Watchdog summary'),
    ];
    foreach ($data['raw'] as $key => $value) {
      $destination = 'others';
      $severity_class = 'ok';
      if (!empty($value['warning'])) {
        $destination = 'warning';
        $severity_class = 'warning';
      }
      if (!empty($value['emergency']) || !empty($value['alert']) || !empty($value['critical']) || !empty($value['error'])) {
        $destination = 'error';
        $severity_class = 'error';
      }
      $rows[$destination][] = [
        'data' => [
          $key,
          $value['emergency'],
          $value['alert'],
          $value['critical'],
          $value['error'],
          $value['warning'],
          $value['notice'],
          $value['info'],
          $value['debug'],
          $value['total'],
        ],
        'class' => [
          'status-' . $severity_class,
        ],
      ];
    }
    if (!empty($rows['error']) || !empty($rows['warning']) || !empty($rows['others'])) {
      $results = $this->generateResult($header, $rows, [
        'report_title' => $rows['title'],
        'error_title' => $this->t('Errors found'),
        'warning_title' => $this->t('Warnings found'),
        'others_title' => $this->t('Other logs'),
      ]);
    }

    return $results;
  }

  /**
   * Parse the decoded JSON data and return a working render array.
   *
   * @param array $data
   *   Array with the decoded JSON data.
   * @param array $ignore
   *   Array with the ignore list.
   *
   * @return array
   *   The render array.
   */
  public function parseModuleUpdateReportForDisplay($data, $ignore) {
    $statuses = $this->getUpdateStatusValues();
    $ignored_title = $this->t('Ignored');
    $results = [];
    // Parse the data.
    $header = [
      $this->t('Module'),
      $this->t('Current version'),
      $this->t('Advice'),
      $this->t('Status'),
    ];
    $rows = [
      'others' => [],
      'warning' => [],
      'error' => [],
    ];
    foreach ($data as $key => $value) {
      if ($key === 'last_checked') {
        continue;
      }
      $ignore_item = in_array($key, $ignore);
      $title = $value['title'] ?? '';
      $current_version = $value['existing_version'] ?? '';
      $recommended_version = $value['recommended'] ?? '';
      $latest_version = $value['latest_version'] ?? '';
      $value_array = [
        '#markup' => $this->t('Recommended: @recommended<br><em>Latest: @latest</em>', [
          '@recommended' => $recommended_version,
          '@latest' => $latest_version,
        ]),
      ];
      if (!empty($value['also'])) {
        foreach ($value['also'] as $also) {
          $value_array['#markup'] .= '<br><em>Also available: ' . $also . '</em>';
        }
      }
      $status = $value['status'] ?? self::CURRENT;
      $status_label_suffix = $ignore_item ? ' (' . $ignored_title . ')' : '';
      $status_label = ($statuses[$status]['title'] ?? '') . $status_label_suffix;
      $status_class = $ignore_item ? 'ignored' : ($statuses[$status]['status'] ?? '');
      $destination = 'others';
      if ($status === self::NOT_CURRENT || $status === self::NOT_SUPPORTED) {
        $destination = 'warning';
      }
      if ($status === self::REVOKED || $status === self::NOT_SECURE) {
        $destination = 'error';
      }
      $rows[$destination][] = [
        'data' => [
          $title . " (" . $key . ")",
          $current_version,
          $this->renderer->render($value_array),
          $status_label,
        ],
        'class' => [
          'status-' . $status_class,
        ],
      ];

    }

    $results['status'] = $this->generateResult($header, $rows, [
      'report_title' => $this->t('Module update report'),
      'error_title' => $this->t('Urgent updates found'),
      'warning_title' => $this->t('Updates found'),
      'others_title' => $this->t('Other modules'),
    ]);

    return $results;
  }

  /**
   * Generate a render array with results.
   *
   * @param array $header
   *   The table header.
   * @param array $rows
   *   The table rows.
   * @param array $titles
   *   The titles of the various categories.
   * @param array $extra
   *   Extra information to add to the result.
   *
   * @return array
   *   The render array.
   */
  private function generateResult($header, $rows, $titles, $extra = []) {
    $result = [];
    $result = [
      '#type' => 'details',
      '#title' => $titles['report_title'],
    ];

    if (!empty($rows['error']) && count($rows['error']) > 0) {
      $result[] = $this->generateResultTable($header, $rows['error'], $titles['error_title'], 'error');
    }

    if (!empty($rows['warning']) && count($rows['warning']) > 0) {
      $result[] = $this->generateResultTable($header, $rows['warning'], $titles['warning_title'], 'warning');
    }

    if (!empty($rows['others']) && count($rows['others']) > 0) {
      $result[] = $this->generateResultTable($header, $rows['others'], $titles['others_title'], 'others');
    }
    if (!empty($extra)) {
      $result[] = $extra;
    }

    return $result;
  }

  /**
   * Generate a table with the results.
   *
   * @param array $header
   *   The table header.
   * @param array $rows
   *   The table rows.
   * @param string $title
   *   The title of the table.
   * @param string $class
   *   The class of the table.
   *
   * @return array
   *   The table render array.
   */
  private function generateResultTable(array $header, array $rows, string $title, string $class) {
    $table = [];
    $table[] = [
      '#markup' => sprintf('<h2>%s</h2>', $title),
    ];
    $table[] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['class' => 'ro-table ro-table-' . $class],
    ];
    return $table;
  }

  /**
   * Get the severities.
   */
  private function getSeverities() {
    return [
      self::REQUIREMENT_INFO => [
        'title' => $this->t('Checked', [], [
          'context' => 'Examined',
        ]),
        'status' => 'checked',
      ],
      self::REQUIREMENT_OK => [
        'title' => $this->t('OK'),
        'status' => 'ok',
      ],
      self::REQUIREMENT_WARNING => [
        'title' => $this->t('Warnings found'),
        'status' => 'warning',
      ],
      self::REQUIREMENT_ERROR => [
        'title' => $this->t('Errors found'),
        'status' => 'error',
      ],
    ];
  }

  /**
   * Get the update statuses.
   */
  private function getUpdateStatusValues() {
    return [
      self::NOT_SECURE => [
        'title' => $this->t('Not secure'),
        'status' => 'error',
      ],
      self::REVOKED => [
        'title' => $this->t('Revoked'),
        'status' => 'error',
      ],
      self::NOT_SUPPORTED => [
        'title' => $this->t('Not supported'),
        'status' => 'warning',
      ],
      self::NOT_CURRENT => [
        'title' => $this->t('New version available'),
        'status' => 'warning',
      ],
      self::CURRENT => [
        'title' => $this->t('Current version'),
        'status' => 'ok',
      ],
    ];
  }

}
