<?php

namespace Drupal\ro_dashboard\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\ro_dashboard\Entity\RoDashboardSite;

/**
 * Service to collect data from a site.
 */
class SiteInfoUpdateService {

  /**
   * Site Guardian update service.
   *
   * @var \Drupal\ro_dashboard\Service\SiteGuardianService
   */
  protected $siteGuardianService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Update sites job queue.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * Logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannel
   */
  protected $loggerChannel;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    SiteGuardianService $site_guardian_service,
    EntityTypeManagerInterface $entity_type_manager,
    QueueFactory $queueFactory,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->siteGuardianService = $site_guardian_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->queue = $queueFactory->get('ro_dashboard_update_sites_job');
    $this->loggerChannel = $logger_factory->get('ro_dashboard');
  }

  /**
   * Update site.
   */
  public function scheduleSiteUpdate() {
    $sites_to_update = $this->entityTypeManager
      ->getStorage('ro_dashboard_site')
      ->loadByProperties(['status' => TRUE]);
    foreach ($sites_to_update as $site_to_update) {
      $queueItem = [];
      $queueItem['site_id'] = $site_to_update->id();
      $this->queue->createItem($queueItem);
    }
    $this->loggerChannel
      ->info('Added @sites to the update-queue.', [
        '@sites' => count($sites_to_update),
      ]);
  }

  /**
   * Update site.
   *
   * @param \Drupal\ro_dashboard\Entity\RoDashboardSite $site_to_update
   *   The site entity to update.
   */
  public function updateSite(RoDashboardSite $site_to_update) {
    $sg_status_report = $site_to_update->get('sg_status_report')->value ?? FALSE;
    $sg_status_report_ignore = $site_to_update->getSgStatusReportIgnore() ?? [];
    $sg_module_update_report = $site_to_update->get('sg_module_update_report')->value ?? FALSE;
    $sg_module_update_report_ignore = $site_to_update->getSgModuleUpdateReportIgnore() ?? [];
    $site_url = $site_to_update->getSiteUrl();
    $site_token = $site_to_update->get('sg_token')->value;
    $results = [
      'data' => [],
      'info' => [
        'error' => 0,
        'warning' => 0,
        'php_version' => '',
        'drupal_version' => '',
        'connection' => '',
      ],
    ];
    $global_status = [];

    if ($sg_status_report && !empty($site_url) && !empty($site_token)) {
      // Get the SiteGuardian status report.
      $scan_results = $this->siteGuardianService->getReport('status', $site_url, $site_token, $sg_status_report_ignore);
      $global_status = $this->checkConnectionError($site_to_update, $scan_results, $global_status);

      if (!empty($scan_results['data'])) {
        $this->parseScanData('sg_status', 'STATUS_ERROR', 'STATUS_WARNING', $scan_results, $results, $global_status);
        $results['info']['php_version'] = $scan_results['info']['php_version'];
        $results['info']['drupal_version'] = $scan_results['info']['drupal_version'];
      }
    }
    if ($sg_module_update_report && !empty($site_url) && !empty($site_token)) {
      // Get the SiteGuardian module updates report.
      $scan_results = $this->siteGuardianService->getReport('modules', $site_url, $site_token, $sg_module_update_report_ignore);
      $global_status = $this->checkConnectionError($site_to_update, $scan_results, $global_status);

      if (!empty($scan_results['data'])) {
        $this->parseScanData('sg_modules', 'SECURITY', 'UPDATES', $scan_results, $results, $global_status);
      }
    }

    if (!empty($results['data'])) {
      $info = $results['info'];

      if (empty($global_status)) {
        array_push($global_status, 'OK');
      }
      $site_to_update->set('global_status', $global_status);
      $site_to_update->set('last_check', date('Y-m-d\\TH:i:s'));
      $site_to_update->set('drupal_version', $info['drupal_version']);
      $site_to_update->set('php_version', $info['php_version']);
      $site_to_update->setReport($results['data']);

      return TRUE;
    }
    return FALSE;
  }

  /**
   * Check if the connection is working.
   *
   * @param \Drupal\ro_dashboard\Entity\RoDashboardSite $site_to_update
   *   The site entity to update.
   * @param array $scan_results
   *   The results of the scan.
   * @param array $global_status
   *   The global status of the site.
   *
   * @return array
   *   The global status of the site.
   */
  private function checkConnectionError(RoDashboardSite $site_to_update, array $scan_results, array $global_status) {
    if (!empty($scan_results['info']['connection']) && $scan_results['info']['connection'] === 'CONNECTION_ERROR') {
      if (!in_array('CONNECTION_ERROR', $global_status)) {
        array_push($global_status, 'CONNECTION_ERROR');
      }
    }
    else {
      unset($global_status['CONNECTION_ERROR']);
    }
    $site_to_update->set('global_status', $global_status);
    $site_to_update->save();

    return $global_status;
  }

  /**
   * Parse the scan data.
   *
   * @param string $key
   *   The key to store the data.
   * @param string $error_status_key
   *   The error status key.
   * @param string $warning_status_key
   *   The warning status key.
   * @param array $scan_results
   *   The results of the scan.
   * @param array $results
   *   The results array.
   * @param array $global_status
   *   The global status array.
   */
  private function parseScanData(
    string $key,
    string $error_status_key,
    string $warning_status_key,
    array $scan_results,
    array &$results,
    array &$global_status,
  ) {
    $results['data'][$key] = $scan_results['data'];
    $results['info']['error'] += $scan_results['info']['error'];
    $results['info']['warning'] += $scan_results['info']['warning'];
    if (!empty($scan_results['info']['error'])) {
      array_push($global_status, $error_status_key);
    }
    if (!empty($scan_results['info']['warning'])) {
      array_push($global_status, $warning_status_key);
    }
  }

}
